from BaseHTTPServer import HTTPServer
from VirtualDeviceHandler import VirtualDeviceHandler
from tinker_access_client.ClientLogger import ClientLogger
from tinker_access_client.ClientOptionParser import ClientOptionParser, ClientOption


# noinspection PyClassHasNoInit
class VirtualDeviceServer:

    @staticmethod
    def start():
        logger = ClientLogger.setup()
        debug_port = ClientOptionParser().parse_args()[0].get(ClientOption.DEBUG_PORT)
        try:
            server = HTTPServer(('', debug_port), VirtualDeviceHandler)
            msg = '\033[92m' \
                  '********************************************************************************\n' \
                  'Virtual Device Initialized, Go To: http://localhost:{0}                         \n' \
                  '********************************************************************************' \
                  '\033[0m'.format(debug_port)
            logger.info(msg)
            server.serve_forever()

        except KeyboardInterrupt:
            server.socket.close()







import os
import sys
import signal
import time
from subprocess import \
    check_output, \
    CalledProcessError

from State import State
from Client import Client
from Command import Command
from daemonize import Daemonize
from PackageInfo import PackageInfo
from ClientOption import ClientOption
from ClientLogger import ClientLogger
from LoggedRequest import LoggedRequest
from CommandExecutor import CommandExecutor


# noinspection PyClassHasNoInit
class ClientDaemon:

    @staticmethod
    def start(opts, args):
        logger = ClientLogger.setup()
        if not ClientDaemon.__status(opts, args):
            try:
                pid_file = opts.get(ClientOption.PID_FILE)
                foreground = opts.get(ClientOption.DEBUG)

                def start():
                    Client.run(opts, args)

                daemon = Daemonize(
                    app=PackageInfo.pip_package_name,
                    pid=pid_file,
                    action=start,
                    foreground=foreground,
                    verbose=True,
                    logger=logger,
                    auto_close_fds=False
                )
                daemon.start()
            except Exception as e:
                msg = '{0} start failed.'.format(PackageInfo.pip_package_name)
                logger.debug(msg)
                logger.exception(e)
                sys.stdout.write(msg + '\n')
                sys.stdout.flush()
                sys.exit(1)
        else:
            sys.stdout.write('{0} is already running...\n'.format(PackageInfo.pip_package_name))
            sys.stdout.flush()
            sys.exit(1)

    @staticmethod
    def stop(opts, args):
        logger = ClientLogger.setup()
        pid_file = opts.get(ClientOption.PID_FILE)
        logout_coast_time = opts.get(ClientOption.LOGOUT_COAST_TIME)
        max_power_down_timeout = opts.get(ClientOption.MAX_POWER_DOWN_TIMEOUT)

        try:
            for process_id in ClientDaemon.__get_process_ids():
                try:
                    os.kill(process_id, signal.SIGTERM)
                except Exception as e:
                    logger.exception(e)
        except CalledProcessError:
            pass

        # Wait for what we assume will be a graceful exit,
        # this will exit early if the pid file is removed as expected.
        current = time.time()
        max_wait_time = max_power_down_timeout + logout_coast_time + 5
        while time.time() - current < max_wait_time and os.path.isfile(pid_file):
            time.sleep(0.1)

        # This should never happen, but... if the pid file still exist at this point, we will nuke it from orbit!
        # Otherwise it will prevent the client daemon from starting again...
        if os.path.isfile(pid_file):
            try:
                os.remove(pid_file)
            except Exception as e:
                logger.exception(e)

    @staticmethod
    def update(opts, args):
        if not ClientDaemon.__is_in_use(opts, args):
            logger = ClientLogger.setup()
            requested_version = args[1] if len(args) >= 2 else None
            if ClientDaemon.__should_update(opts, requested_version):
                try:
                    requested_package = PackageInfo.pip_package_name

                    # BUG: There is a big fat bug with pip that is causing it to redirect to
                    # install the latest version, even when a specific version is installed.
                    # I'll look into this when I get time.

                    if requested_version:
                        requested_package = '{0}=={1}'.format(requested_package, requested_version)

                    ClientDaemon.stop(opts, args)
                    CommandExecutor().execute_commands([
                        'pip install --upgrade --force-reinstall --ignore-installed --no-cache-dir {0}'
                        .format(requested_package)
                    ])
                except Exception as e:
                    msg = '{0} update failed, remediation maybe required!'.format(PackageInfo.pip_package_name)
                    logger.debug(msg)
                    logger.exception(e)
                    sys.stdout.write(msg + '\n')
                    sys.stdout.flush()
                    sys.exit(1)
                finally:
                    if not ClientDaemon.__status(opts, args):
                        ClientDaemon.restart(opts, args)
            else:
                version = 'current' if not requested_version else 'requested'
                sys.stdout.write(
                    '{0} v{1} already matches the {2} version. \n'
                    'Use the --force-update option to bypass the version check and re-install.\n'.format(
                        PackageInfo.pip_package_name,
                        PackageInfo.version,
                        version
                    ))
                sys.stdout.flush()
                sys.exit(1)
        else:
            sys.stdout.write('{0} is currently in use, try again later...\n'.format(PackageInfo.pip_package_name))
            sys.stdout.flush()
            sys.exit(1)

    @staticmethod
    def __should_update(opts, requested_version=None):
        logger = ClientLogger.setup()
        try:
            if opts.get(ClientOption.FORCE_UPDATE):
                return True

            response = LoggedRequest.get('https://pypi.python.org/pypi/{0}/json'.format(PackageInfo.pip_package_name))
            response.raise_for_status()
            data = response.json()

            current_version = PackageInfo.version
            latest_version = data.get('info', {}).get('version', '0.0.0')
            latest_version = '.'.join([s.zfill(2) for s in latest_version.split('.')])
            if not requested_version:
                return current_version != latest_version

            requested_version = '.'.join([s.zfill(2) for s in requested_version.split('.')])
            return current_version != requested_version
        except Exception as e:
            logger.exception(e)
        return False

    @staticmethod
    def restart(opts, args):
        logger = ClientLogger.setup()
        reboot_delay = opts.get(ClientOption.REBOOT_DELAY)
        try:
            args[0] = Command.STOP.get('command')
            ClientDaemon.stop(opts, args)
            logger.debug('Restarting in %s seconds...', reboot_delay)
            time.sleep(reboot_delay)
            args[0] = Command.START.get('command')
            ClientDaemon.start(opts, args)
        except Exception as e:
            msg = '{0} restart failed.'.format(PackageInfo.pip_package_name)
            logger.debug(msg)
            logger.exception(e)
            sys.stdout.write(msg + '\n')
            sys.stdout.flush()
            sys.exit(1)
        finally:
            if not ClientDaemon.__status(opts, args):
                ClientDaemon.restart(opts, args)

    @staticmethod
    def remove(opts, args):
        try:
            CommandExecutor().execute_commands([
                '{'
                '\tset +e',
                '\tservice tinker-access-client stop',
                '\tupdate-rc.d -f tinker-access-client remove',
                '\trm -rf /etc/init.d/tinker-access-client',
                '\twhile [[ $(pip uninstall tinker-access-client -y) == 0 ]]; do :; done',
                '\tfind /usr/local/lib/python2.7/dist-packages/ -name \'tinker[_-]access[_-]client*\' -type d -exec sudo rm -r "{}" \;',
                '\tsed -i.bak \'/^\/.*\/tinker_access_client$/s///g\' /usr/local/lib/python2.7/dist-packages/easy-install.pth',
                '} &> /dev/null'
            ])
        except Exception:
            pass

    @staticmethod
    def status(opts, args):
        status = ClientDaemon.__status(opts, args)
        if status:
            sys.stdout.write('{0}\n'.format(status))
            sys.stdout.flush()
            sys.exit(0)
        else:
            sys.stdout.write('{0}\n'.format(State.TERMINATED))
            sys.stdout.flush()
            sys.exit(1)

    @staticmethod
    def __status(opts, _):
        process_ids = ClientDaemon.__get_process_ids()
        status_file = opts.get(ClientOption.STATUS_FILE)

        status = terminated = State.TERMINATED
        if os.path.isfile(status_file):
            with open(status_file, 'r') as f:
                status = f.readline().strip()

        return status if status is not terminated and len(process_ids) > 0 else None

    @staticmethod
    def __is_in_use(opts, args):
        status = ClientDaemon.__status(opts, args)
        return status == State.IN_USE or status == State.IN_TRAINING

    @staticmethod
    def __get_process_ids():
        process_ids = []

        try:
            cmd = ['pgrep', '-f', '(/{0}\s+(start|restart|update))'.format(PackageInfo.pip_package_name)]
            with open(os.devnull, 'w') as devnull:
                for process_id in check_output(cmd, stderr=devnull).splitlines():
                    process_id = int(process_id)
                    is_current_process = process_id == int(os.getpid())
                    if not is_current_process:
                        process_ids.append(process_id)
        except CalledProcessError:
            pass

        return process_ids



#!/usr/bin/env python

# The wiring for the LCD is as follows:
# 1 : GND
# 2 : 5V
# 3 : Contrast (0-5V)*      Tie to Gnd for max contrast
# 4 : RS (Register Select)
# 5 : R/W (Read Write)       - GROUND THIS PIN
# 6 : Enable or Strobe
# 7 : Data Bit 0             - NOT USED
# 8 : Data Bit 1             - NOT USED
# 9 : Data Bit 2             - NOT USED
# 10: Data Bit 3             - NOT USED
# 11: Data Bit 4
# 12: Data Bit 5
# 13: Data Bit 6
# 14: Data Bit 7
# 15: LCD Backlight +5V**
# 16: LCD Backlight GND

import RPi
import time

# Define GPIO to LCD mapping
LCD_RS = 7  # 4 : RS (Register Select)  [RPi pin 7 is connected to PiWedge 'CE1' ]
LCD_E  = 8  # 6 : Enable or Strobe      [RPi pin 8 is connected to PiWedge 'CE0' ]
LCD_D4 = 25 # 11: Data Bit 4
LCD_D5 = 24 # 12: Data Bit 5
LCD_D6 = 23 # 13: Data Bit 6
LCD_D7 = 18 # 14: Data Bit 7

# Define some device constants
LCD_WIDTH = 16    # Maximum characters per line
LCD_CHR = True
LCD_CMD = False

LCD_LINE_1 = 0x80 # LCD RAM address for the 1st line
LCD_LINE_2 = 0xC0 # LCD RAM address for the 2nd line

# Timing constants
E_PULSE = 0.0005
E_DELAY = 0.0005

def main():
  # Main program block
  print("lcdModule main")

  # Initialise display
  lcd_init()


def lcd_init():
  GPIO = RPi.GPIO
  GPIO.setmode(GPIO.BCM)       # Use BCM GPIO numbers
  GPIO.setup(LCD_E, GPIO.OUT)  # E
  GPIO.setup(LCD_RS, GPIO.OUT) # RS
  GPIO.setup(LCD_D4, GPIO.OUT) # DB4
  GPIO.setup(LCD_D5, GPIO.OUT) # DB5
  GPIO.setup(LCD_D6, GPIO.OUT) # DB6
  GPIO.setup(LCD_D7, GPIO.OUT) # DB7

  # Initialise display
  lcd_byte(0x33,LCD_CMD) # 110011 Initialise
  lcd_byte(0x32,LCD_CMD) # 110010 Initialise
  lcd_byte(0x06,LCD_CMD) # 000110 Cursor move direction
  lcd_byte(0x0C,LCD_CMD) # 001100 Display On,Cursor Off, Blink Off
  lcd_byte(0x28,LCD_CMD) # 101000 Data length, number of lines, font size
  lcd_byte(0x01,LCD_CMD) # 000001 Clear display
  time.sleep(E_DELAY)

def lcd_byte(bits, mode):
  # Send byte to data pins
  # bits = data
  # mode = True  for character
  #        False for command

  GPIO = RPi.GPIO
  GPIO.output(LCD_RS, mode) # RS

  # High bits
  GPIO.output(LCD_D4, False)
  GPIO.output(LCD_D5, False)
  GPIO.output(LCD_D6, False)
  GPIO.output(LCD_D7, False)
  if bits&0x10==0x10:
    GPIO.output(LCD_D4, True)
  if bits&0x20==0x20:
    GPIO.output(LCD_D5, True)
  if bits&0x40==0x40:
    GPIO.output(LCD_D6, True)
  if bits&0x80==0x80:
    GPIO.output(LCD_D7, True)

  lcd_toggle_enable()

  # Low bits
  GPIO.output(LCD_D4, False)
  GPIO.output(LCD_D5, False)
  GPIO.output(LCD_D6, False)
  GPIO.output(LCD_D7, False)
  if bits&0x01==0x01:
    GPIO.output(LCD_D4, True)
  if bits&0x02==0x02:
    GPIO.output(LCD_D5, True)
  if bits&0x04==0x04:
    GPIO.output(LCD_D6, True)
  if bits&0x08==0x08:
    GPIO.output(LCD_D7, True)

  lcd_toggle_enable()

def lcd_toggle_enable():
  GPIO = RPi.GPIO
  time.sleep(E_DELAY)
  GPIO.output(LCD_E, True)
  time.sleep(E_PULSE)
  GPIO.output(LCD_E, False)
  time.sleep(E_DELAY)

def lcd_string(message,line):
  # Send string to display
  lcd_byte(line, LCD_CMD)

  if (len(message) > LCD_WIDTH) and 0:
    for i in range(len(message)-LCD_WIDTH + 1):
      time.sleep(3)
      lcd_byte(line, LCD_CMD)
      for c in message[i:(len(message))]:
        lcd_byte(ord(c),LCD_CHR)

  else:
    messageLJ = message.ljust(LCD_WIDTH," ")
    for i in range(LCD_WIDTH):
      lcd_byte(ord(messageLJ[i]),LCD_CHR)





from PackageInfo import PackageInfo


class Command(object):

    START = dict(
        command='start',
        description='This command will start the {0}.\n'.format(PackageInfo.pip_package_name)
    )
    STOP = dict(
        command='stop',
        description='This command will stop the {0}.\n'.format(PackageInfo.pip_package_name)
    )
    STATUS = dict(
        command='status',
        description='This command will return the current state of the {0}\n'
                    '\t\t (i.e. initialized, idle, in_use, in_training, terminated)\n'
                    .format(PackageInfo.pip_package_name)
    )
    RESTART = dict(
        command='restart',
        description='This command will stop the {0}.\n'.format(PackageInfo.pip_package_name)
    )
    UPDATE = dict(
        command='update',
        description='This command will update the {0} using PyPI - the Python Package Index,\n'
        '\t\t by default the latest published version will be installed.\n'
        '\t\t Optionally a second argument can be provided to specify the specific version desired. \n'
        '\t\t (i.e. \'sudo {0} update 2017.2.14.441\')\n'.format(PackageInfo.pip_package_name)
    )
    REMOVE = dict(
        command='remove',
        description='This command will stop the {0} service.\n '
        '\t\t prevent it from starting on next boot and completely\n'
        '\t\t remove all artifacts installed via PIP'
        .format(PackageInfo.pip_package_name)
    )
    # RELOAD = dict(command='reload', description='reload...')
    # REMOVE = dict(command='remove', description='remove...')
    # UNINSTALL = dict(command='uninstall', description='uninstall...')
    # RECONFIGURE = dict(command='re-configure', description='re-configure...')
    # FORCE_RELOAD = dict(command='force_reload', description='force_reload...')

    def __new__(self, command):
        for key, value in vars(Command).items():
            if not key.startswith('__'):
                if command == value['command']:
                    return value

        return None



import time
import logging


# The wiring for the LCD is as follows:
# 1 : GND
# 2 : 5V
# 3 : Contrast (0-5V)*      Tie to Gnd for max contrast
# 4 : RS (Register Select)
# 5 : R/W (Read Write)       - GROUND THIS PIN
# 6 : Enable or Strobe
# 7 : Data Bit 0             - NOT USED
# 8 : Data Bit 1             - NOT USED
# 9 : Data Bit 2             - NOT USED
# 10: Data Bit 3             - NOT USED
# 11: Data Bit 4
# 12: Data Bit 5
# 13: Data Bit 6
# 14: Data Bit 7
# 15: LCD Backlight +5V**
# 16: LCD Backlight GND

# Define GPIO to LCD mapping
LCD_RS = 7  # 4 : RS (Register Select)  [RPi pin 7 is connected to PiWedge 'CE1' ]
LCD_E = 8  # 6 : Enable or Strobe      [RPi pin 8 is connected to PiWedge 'CE0' ]
LCD_D4 = 25  # 11: Data Bit 4
LCD_D5 = 24  # 12: Data Bit 5
LCD_D6 = 23  # 13: Data Bit 6
LCD_D7 = 18  # 14: Data Bit 7

# Define some device constants
LCD_WIDTH = 16  # Maximum characters per line
LCD_CHR = True
LCD_CMD = False

LCD_LINE_1 = 0x80  # LCD RAM address for the 1st line
LCD_LINE_2 = 0xC0  # LCD RAM address for the 2nd line

# Timing constants
E_PULSE = 0.0005
E_DELAY = 0.0005


class LcdApi(object):

    def __init__(self, gpio):
        self.__logger = logging.getLogger(__name__)

        # noinspection PyPep8Naming
        GPIO = self.__GPIO = gpio
        GPIO.setup(LCD_E, GPIO.OUT)  # E
        GPIO.setup(LCD_RS, GPIO.OUT)  # RS
        GPIO.setup(LCD_D4, GPIO.OUT)  # DB4
        GPIO.setup(LCD_D5, GPIO.OUT)  # DB5
        GPIO.setup(LCD_D6, GPIO.OUT)  # DB6
        GPIO.setup(LCD_D7, GPIO.OUT)  # DB7

        self.__init__display()

    def __init__display(self):
        # # Initialise display
        self.lcd_byte(0x33, LCD_CMD)  # 110011 Initialise
        self.lcd_byte(0x32, LCD_CMD)  # 110010 Initialise
        self.lcd_byte(0x06, LCD_CMD)  # 000110 Cursor move direction
        self.lcd_byte(0x0C, LCD_CMD)  # 001100 Display On,Cursor Off, Blink Off
        self.lcd_byte(0x28, LCD_CMD)  # 101000 Data length, number of lines, font size
        self.lcd_byte(0x01, LCD_CMD)  # 000001 Clear display
        time.sleep(E_DELAY)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def lcd_byte(self, bits, mode):

        # Send byte to data pins
        # bits = data
        # mode = True  for character
        #        False for command

        # noinspection PyPep8Naming
        GPIO = self.__GPIO
        GPIO.output(LCD_RS, mode)  # RS --Select Register


        # High bits
        GPIO.output(LCD_D4, False)
        GPIO.output(LCD_D5, False)
        GPIO.output(LCD_D6, False)
        GPIO.output(LCD_D7, False)
        if bits & 0x10 == 0x10:
            GPIO.output(LCD_D4, True)
        if bits & 0x20 == 0x20:
            GPIO.output(LCD_D5, True)
        if bits & 0x40 == 0x40:
            GPIO.output(LCD_D6, True)
        if bits & 0x80 == 0x80:
            GPIO.output(LCD_D7, True)

        self.lcd_toggle_enable()

        # Low bits
        GPIO.output(LCD_D4, False)
        GPIO.output(LCD_D5, False)
        GPIO.output(LCD_D6, False)
        GPIO.output(LCD_D7, False)
        if bits & 0x01 == 0x01:
            GPIO.output(LCD_D4, True)
        if bits & 0x02 == 0x02:
            GPIO.output(LCD_D5, True)
        if bits & 0x04 == 0x04:
            GPIO.output(LCD_D6, True)
        if bits & 0x08 == 0x08:
            GPIO.output(LCD_D7, True)

        self.lcd_toggle_enable()

    def lcd_toggle_enable(self):

        # noinspection PyPep8Naming
        GPIO = self.__GPIO
        time.sleep(E_DELAY)
        GPIO.output(LCD_E, True)
        time.sleep(E_PULSE)
        GPIO.output(LCD_E, False)
        time.sleep(E_DELAY)

    def lcd_string(self, message, line):

        # Send string to display
        self.lcd_byte(line, LCD_CMD)

        if (len(message) > LCD_WIDTH) and 0:
            for i in range(len(message) - LCD_WIDTH + 1):
                time.sleep(3)
                self.lcd_byte(line, LCD_CMD)
                for c in message[i:(len(message))]:
                    self.lcd_byte(ord(c), LCD_CHR)

        else:
            messageLJ = message.ljust(LCD_WIDTH, " ")
            for i in range(LCD_WIDTH):
                self.lcd_byte(ord(messageLJ[i]), LCD_CHR)

    def write(self, first_line, second_line):
        self.lcd_string(first_line, LCD_LINE_1)
        self.lcd_string(second_line, LCD_LINE_2)
